@@ -3,7 +3,7 @@ package Net::DNS::RR;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: RR.pm 1910 2023-03-30 19:16:30Z willem $)[2];
+our $VERSION = (qw$Id: RR.pm 2028 2025-07-22 16:52:42Z willem $)[2];
 
 
 =head1 NAME
@@ -12,15 +12,15 @@ Net::DNS::RR - DNS resource record base class
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
+	use Net::DNS;
 
-    $rr = Net::DNS::RR->new('example.com IN AAAA 2001:DB8::1');
+	$rr = Net::DNS::RR->new('example.com IN AAAA 2001:DB8::1');
 
-    $rr = Net::DNS::RR->new(
-	    owner   => 'example.com',
-	    type    => 'AAAA',
-	    address => '2001:DB8::1'
-	    );
+	$rr = Net::DNS::RR->new(
+			owner	=> 'example.com',
+			type	=> 'AAAA',
+			address => '2001:DB8::1'
+			);
 
 
 =head1 DESCRIPTION
@@ -64,10 +64,10 @@ sub new {
 
 =head2 new (from string)
 
-    $aaaa  = Net::DNS::RR->new('host.example.com. 86400 AAAA 2001:DB8::1');
-    $mx	   = Net::DNS::RR->new('example.com. 7200 MX 10 mailhost.example.com.');
-    $cname = Net::DNS::RR->new('www.example.com 300 IN CNAME host.example.com');
-    $txt   = Net::DNS::RR->new('txt.example.com 3600 HS TXT "text data"');
+	$aaaa	= Net::DNS::RR->new('host.example.com. 86400 AAAA 2001:DB8::1');
+	$mx	= Net::DNS::RR->new('example.com. 7200 MX 10 mailhost.example.com.');
+	$cname	= Net::DNS::RR->new('www.example.com 300 IN CNAME host.example.com');
+	$txt	= Net::DNS::RR->new('txt.example.com 3600 HS TXT "text data"');
 
 Returns an object of the appropriate RR type, or a L<Net::DNS::RR> object
 if the type is not implemented. The attribute values are extracted from the
@@ -89,11 +89,11 @@ my $PARSE_REGEX = q/("[^"]*")|;[^\n]*|[ \t\n\r\f()]+/;		# NB: *not* \s (matches
 
 sub _new_string {
 	my ( $base, $string ) = @_;
-	local $_ = $string;
-	die 'argument absent or undefined' unless defined $_;
-	die 'non-scalar argument' if ref $_;
+	die 'argument absent or undefined' unless defined $string;
+	die 'non-scalar argument' if ref $string;
 
 	# parse into quoted strings, contiguous non-whitespace and (discarded) comments
+	local $_ = $string;
 	s/\\\\/\\092/g;						# disguise escaped escape
 	s/\\"/\\034/g;						# disguise escaped quote
 	s/\\\(/\\040/g;						# disguise escaped bracket
@@ -143,21 +143,21 @@ sub _new_string {
 
 =head2 new (from hash)
 
-    $rr = Net::DNS::RR->new(%hash);
+	$rr = Net::DNS::RR->new(%hash);
 
-    $rr = Net::DNS::RR->new(
-	    owner   => 'host.example.com',
-	    ttl	    => 86400,
-	    class   => 'IN',
-	    type    => 'AAAA',
-	    address => '2001:DB8::1'
-	    );
- 
-    $rr = Net::DNS::RR->new(
-	    owner   => 'txt.example.com',
-	    type    => 'TXT',
-	    txtdata => [ 'one', 'two' ]
-	    );
+	$rr = Net::DNS::RR->new(
+			owner	=> 'host.example.com',
+			ttl	=> 86400,
+			class	=> 'IN',
+			type	=> 'AAAA',
+			address => '2001:DB8::1'
+			);
+	
+	$rr = Net::DNS::RR->new(
+			owner	=> 'txt.example.com',
+			type	=> 'TXT',
+			txtdata => [ 'one', 'two' ]
+			);
 
 Returns an object of the appropriate RR type, or a L<Net::DNS::RR> object
 if the type is not implemented. Consult the relevant manual pages for the
@@ -174,20 +174,24 @@ my @core = qw(owner name type class ttl rdlength);
 sub _new_hash {
 	my $base = shift;
 
-	my %attribute = ( owner => '.', type => 'NULL' );
+	my %argument = ( owner => '.', type => 'NULL' );
+	my @attribute;
 	while ( my $key = shift ) {
-		$attribute{lc $key} = shift;
+		push @attribute, $key;
+		$argument{lc $key} = shift;
 	}
 
-	my ( $owner, $name, $type, $class, $ttl ) = delete @attribute{@core};
+	my ( $owner, $name, $type, $class, $ttl ) = delete @argument{@core};
 
-	my $self = $base->_subclass( $type, scalar(%attribute) );
+	my $self = $base->_subclass( $type, scalar(%argument) );
 	$self->owner( $name ? $name : $owner );
 	$self->class($class) if defined $class;			# optional CLASS
 	$self->ttl($ttl)     if defined $ttl;			# optional TTL
 
 	eval {
-		while ( my ( $attribute, $value ) = each %attribute ) {
+		foreach my $attribute (@attribute) {
+			my $value = $argument{lc $attribute};
+			next unless defined $value;
 			$self->$attribute( ref($value) eq 'ARRAY' ? @$value : $value );
 		}
 	};
@@ -200,7 +204,7 @@ sub _new_hash {
 
 =head2 decode
 
-    ( $rr, $next ) = Net::DNS::RR->decode( \$data, $offset, @opaque );
+	( $rr, $next ) = Net::DNS::RR->decode( \$data, $offset, @opaque );
 
 Decodes a DNS resource record at the specified location within a
 DNS packet.
@@ -234,8 +238,11 @@ sub decode {
 	my $next = $index + $self->{rdlength};
 	die 'corrupt wire-format data' if length $$data < $next;
 
-	local $self->{offset} = $offset;
-	$self->_decode_rdata( $data, $index, @opaque ) if $next > $index or $self->type eq 'OPT';
+	if ( $next > $index or $self->type eq 'OPT' ) {
+		local $self->{offset} = $offset;
+		eval { $self->_decode_rdata( $data, $index, @opaque ) };
+		warn $@ if $@;
+	}
 
 	return wantarray ? ( $self, $next ) : $self;
 }
@@ -243,7 +250,7 @@ sub decode {
 
 =head2 encode
 
-    $data = $rr->encode( $offset, @opaque );
+	$data = $rr->encode( $offset, @opaque );
 
 Returns the C<Net::DNS::RR> in binary format suitable for inclusion
 in a DNS packet buffer.
@@ -269,14 +276,14 @@ sub encode {
 
 =head2 canonical
 
-    $data = $rr->canonical;
+	$data = $rr->canonical;
 
 Returns the C<Net::DNS::RR> in canonical binary format suitable for
 DNSSEC signature validation.
 
 The absence of the associative array argument signals to subordinate
-encoders that the canonical uncompressed lower case form of embedded
-domain names is to be used.
+encoders that the canonical uncompressed form of embedded domain
+names is to be used.
 
 =cut
 
@@ -292,7 +299,7 @@ sub canonical {
 
 =head2 print
 
-    $rr->print;
+	$rr->print;
 
 Prints the resource record to the currently selected output filehandle.
 Calls the string method to get the formatted RR representation.
@@ -307,7 +314,7 @@ sub print {
 
 =head2 string
 
-    print $rr->string, "\n";
+	print $rr->string, "\n";
 
 Returns a string representation of the RR using the master file format
 mandated by RFC1035.
@@ -328,20 +335,20 @@ sub string {
 	my @rdata = $empty ? () : eval { $self->_format_rdata };
 	carp $@ if $@;
 
-	my $tab = length($name) < 72 ? "\t" : ' ';
+	my $tab	 = length($name) < 72 ? "\t" : ' ';
 	my @line = _wrap( join( $tab, @core, '(' ), @rdata, ')' );
 
 	my $last = pop(@line);					# last or only line
 	$last = join $tab, @core, "@rdata" unless scalar(@line);
 
 	$self->_annotation('no data') if $empty;
-	return join "\n\t", @line, _wrap( $last, map {"; $_"} $self->_annotation );
+	return join "\n\t", @line, _wrap( $last, $self->_annotation );
 }
 
 
 =head2 plain
 
-    $plain = $rr->plain;
+	$plain = $rr->plain;
 
 Returns a simplified single-line representation of the RR.
 This facilitates interaction with programs like nsupdate
@@ -356,7 +363,7 @@ sub plain {
 
 =head2 token
 
-    @token = $rr->token;
+	@token = $rr->token;
 
 Returns a token list representation of the RR zone file string.
 
@@ -381,7 +388,7 @@ sub token {
 
 =head2 generic
 
-    $generic = $rr->generic;
+	$generic = $rr->generic;
 
 Returns the generic RR representation defined in RFC3597. This facilitates
 creation of zone files containing RRs unrecognised by outdated nameservers
@@ -405,7 +412,7 @@ sub generic {
 
 =head2 owner name
 
-    $name = $rr->owner;
+	$name = $rr->owner;
 
 Returns the owner name of the record.
 
@@ -422,7 +429,7 @@ sub name { return &owner; }		## historical
 
 =head2 type
 
-    $type = $rr->type;
+	$type = $rr->type;
 
 Returns the record type.
 
@@ -437,7 +444,7 @@ sub type {
 
 =head2 class
 
-    $class = $rr->class;
+	$class = $rr->class;
 
 Resource record class.
 
@@ -452,8 +459,8 @@ sub class {
 
 =head2 ttl
 
-    $ttl = $rr->ttl;
-    $ttl = $rr->ttl(3600);
+	$ttl = $rr->ttl;
+	$ttl = $rr->ttl(3600);
 
 Resource record time to live in seconds.
 
@@ -533,7 +540,7 @@ sub rdatastr {				## historical RR subtype method
 
 =head2 rdata
 
-    $rr = Net::DNS::RR->new( type => NULL, rdata => 'arbitrary' );
+	$rr = Net::DNS::RR->new( type => NULL, rdata => 'arbitrary' );
 
 Resource record data section when viewed as opaque octets.
 
@@ -552,7 +559,7 @@ sub rdata {
 
 =head2 rdstring
 
-    $rdstring = $rr->rdstring;
+	$rdstring = $rr->rdstring;
 
 Returns a string representation of the RR-specific data.
 
@@ -571,7 +578,7 @@ sub rdstring {
 
 =head2 rdlength
 
-    $rdlength = $rr->rdlength;
+	$rdlength = $rr->rdlength;
 
 Returns the uncompressed length of the encoded RR-specific data.
 
@@ -593,13 +600,13 @@ comparator function used for a particular RR based on its attributes.
 
 =head2 set_rrsort_func
 
-    my $function = sub {	## numerically ascending order
-	$Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'};
-    };
+	my $function = sub {	## numerically ascending order
+		$Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'};
+		};
 
-    Net::DNS::RR::MX->set_rrsort_func( 'preference', $function );
+	Net::DNS::RR::MX->set_rrsort_func( 'preference', $function );
 
-    Net::DNS::RR::MX->set_rrsort_func( 'default_sort', $function );
+	Net::DNS::RR::MX->set_rrsort_func( 'default_sort', $function );
 
 set_rrsort_func() must be called as a class method. The first argument is
 the attribute name on which the sorting is to take place. If you specify
@@ -631,8 +638,8 @@ sub set_rrsort_func {
 
 =head2 get_rrsort_func
 
-    $function = Net::DNS::RR::MX->get_rrsort_func('preference');
-    $function = Net::DNS::RR::MX->get_rrsort_func();
+	$function = Net::DNS::RR::MX->get_rrsort_func('preference');
+	$function = Net::DNS::RR::MX->get_rrsort_func();
 
 get_rrsort_func() returns a reference to the comparator function.
 
@@ -712,8 +719,8 @@ sub _subclass {
 
 sub _annotation {
 	my ( $self, @note ) = @_;
-	$self->{annotation} = ["@note"] if scalar @note;
-	return wantarray ? @{$self->{annotation} || []} : ();
+	push @{$self->{annotation}}, "; @note", "\n" if scalar @note;
+	return wantarray ? @{delete( $self->{annotation} ) || []} : undef;
 }
 
 
@@ -766,16 +773,14 @@ sub AUTOLOAD {				## Default method
 	our $AUTOLOAD;
 	my ($method) = reverse split /::/, $AUTOLOAD;
 
-	for ( my $action = $method ) {	## tolerate mixed-case attribute name
-		tr [A-Z-] [a-z_];
-		if ( $self->can($action) ) {
-			*{$AUTOLOAD} = sub { shift->$action(@_) };
-			return &$AUTOLOAD;
-		}
+	my $canonical = lc($method);	## tolerate mixed-case attribute name
+	if ( $self->can($canonical) ) {
+		*{$AUTOLOAD} = sub { shift->$canonical(@_) };
+		return &$AUTOLOAD;
 	}
 
 	my $oref = ref($self);
-	*{$AUTOLOAD} = sub {};		## suppress deep recursion
+	*{$AUTOLOAD} = sub { };		## suppress deep recursion
 	croak qq[$self has no class method "$method"] unless $oref;
 
 	my $string = $self->string;
@@ -783,7 +788,7 @@ sub AUTOLOAD {				## Default method
 	my $module = join '::', __PACKAGE__, $self->type;
 	eval("require $module") if $oref eq __PACKAGE__;	## no critic ProhibitStringyEval
 
-	@_ = ( <<"END" );
+	@_ = (<<"END");
 ***  FATAL PROGRAM ERROR!!	Unknown instance method "$method"
 ***  which the program has attempted to call for the object:
 ***
@@ -840,8 +845,8 @@ DEALINGS IN THE SOFTWARE.
 
 L<perl> L<Net::DNS>
 L<Net::DNS::Question> L<Net::DNS::Packet> L<Net::DNS::Update>
-L<RFC1035(4.1.3)|https://tools.ietf.org/html/rfc1035>
-L<RFC3597|https://tools.ietf.org/html/rfc3597>
+L<RFC1035(4.1.3)|https://iana.org/go/rfc1035#section-4.1.3>
+L<RFC3597|https://iana.org/go/rfc3597>
 
 =cut
 
