@@ -2,7 +2,7 @@ package Net::DNS::RR::CERT;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: CERT.pm 1896 2023-01-30 12:59:25Z willem $)[2];
+our $VERSION = (qw$Id: CERT.pm 2002 2025-01-07 09:57:46Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -141,6 +141,8 @@ sub tag { return &keytag; }					# uncoverable pod
 		'ECDSAP384SHA384'    => 14,			# [RFC6605]
 		'ED25519'	     => 15,			# [RFC8080]
 		'ED448'		     => 16,			# [RFC8080]
+		'SM2SM3'	     => 17,			# [RFC-cuiling-dnsop-sm2-alg-15]
+		'ECC-GOST12'	     => 23,			# [RFC-makarenko-gost2012-dnssec-05]
 
 		'INDIRECT'   => 252,				# [RFC4034]
 		'PRIVATEDNS' => 253,				# [RFC4034]
@@ -178,8 +180,8 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    $rr = Net::DNS::RR->new('name IN CERT certtype keytag algorithm cert');
+	use Net::DNS;
+	$rr = Net::DNS::RR->new('name IN CERT certtype keytag algorithm cert');
 
 =head1 DESCRIPTION
 
@@ -197,20 +199,20 @@ other unpredictable behaviour.
 
 =head2 certtype
 
-    $certtype = $rr->certtype;
+	$certtype = $rr->certtype;
 
 Returns the certtype code for the certificate (in numeric form).
 
 =head2 keytag
 
-    $keytag = $rr->keytag;
-    $rr->keytag( $keytag );
+	$keytag = $rr->keytag;
+	$rr->keytag( $keytag );
 
 Returns the key tag for the public key in the certificate
 
 =head2 algorithm
 
-    $algorithm = $rr->algorithm;
+	$algorithm = $rr->algorithm;
 
 Returns the algorithm used by the certificate (in numeric form).
 
@@ -218,15 +220,15 @@ Returns the algorithm used by the certificate (in numeric form).
 
 =head2 certbin
 
-    $certbin = $rr->certbin;
-    $rr->certbin( $certbin );
+	$certbin = $rr->certbin;
+	$rr->certbin( $certbin );
 
 Binary representation of the certificate.
 
 =head2 cert
 
-    $cert = $rr->cert;
-    $rr->cert( $cert );
+	$cert = $rr->cert;
+	$rr->cert( $cert );
 
 Base64 representation of the certificate.
 
@@ -262,8 +264,8 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl> L<Net::DNS> L<Net::DNS::RR>
-L<RFC4398|https://tools.ietf.org/html/rfc4398>
+L<RFC4398|https://iana.org/go/rfc4398>
 
-L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
+L<Algorithm Numbers|https://iana.org/assignments/dns-sec-alg-numbers>
 
 =cut
